from conftest import compare_have_want


def test_empty_table():
    compare_have_want(
        have="""\
        :table:

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_one_row_no_header():
    compare_have_want(
        have="""
        :table:

        :tbody:

        :tr: :td:foo:: :td:bar:: ::

        ::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <tbody class="tablebody" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">foo</td>

        <td class="tabledatum" data-nodeid="6">bar</td>

        </tr>

        </tbody>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_two_rows_no_header():
    compare_have_want(
        have="""
        :table:

        :tbody:

        :tr: :td:one:: :td:two:: ::

        :tr: :td:three:: :td:four:: ::

        ::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <tbody class="tablebody" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">one</td>

        <td class="tabledatum" data-nodeid="6">two</td>

        </tr>

        <tr class="tablerow" data-nodeid="8">

        <td class="tabledatum" data-nodeid="9">three</td>

        <td class="tabledatum" data-nodeid="11">four</td>

        </tr>

        </tbody>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_with_header():
    compare_have_want(
        have="""
        :table:

        :thead:

        :tr: :td: head1 :: :td: head2 :: ::

        ::

        :tbody:

        :tr: :td:one:: :td:two:: ::

        :tr: :td:three:: :td:four:: ::

        ::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <thead class="tablehead" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">head1</td>

        <td class="tabledatum" data-nodeid="6">head2</td>

        </tr>

        </thead>

        <tbody class="tablebody" data-nodeid="8">

        <tr class="tablerow" data-nodeid="9">

        <td class="tabledatum" data-nodeid="10">one</td>

        <td class="tabledatum" data-nodeid="12">two</td>

        </tr>

        <tr class="tablerow" data-nodeid="14">

        <td class="tabledatum" data-nodeid="15">three</td>

        <td class="tabledatum" data-nodeid="17">four</td>

        </tr>

        </tbody>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_math_in_header():
    compare_have_want(
        have=r"""
        :table:

        :thead:

        :tr: :td: $H^{\eta}$ :: :td: $H^{*}$ :: :td: $H^{c}$ :: ::

        ::

        ::
        """,
        want=r"""        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <thead class="tablehead" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4"><span class="math" data-nodeid="5">\(H^{\eta}\)</span></td>

        <td class="tabledatum" data-nodeid="7"><span class="math" data-nodeid="8">\(H^{*}\)</span></td>

        <td class="tabledatum" data-nodeid="10"><span class="math" data-nodeid="11">\(H^{c}\)</span></td>

        </tr>

        </thead>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_shortcuts_in_header():
    compare_have_want(
        have="""
        :table:

        :tbody:

        :tr: :td: *foo* :: :td: **bar** :: ::

        ::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <tbody class="tablebody" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4"><span class="span" data-nodeid="5"><em>foo</em></span></td>

        <td class="tabledatum" data-nodeid="7"><span class="span" data-nodeid="8"><strong>bar</strong></span></td>

        </tr>

        </tbody>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_simple_caption():
    compare_have_want(
        have="""
        :table:

        :tbody:

        :tr: :td:foo:: :td:bar:: ::

        ::

        :caption:
        This is a caption.

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <tbody class="tablebody" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">foo</td>

        <td class="tabledatum" data-nodeid="6">bar</td>

        </tr>

        </tbody>

        <caption>
        <span class="label">Table 1. </span>This is a caption.
        </caption>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_simple_caption_with_shortcuts():
    compare_have_want(
        have="""
        :table:

        :tbody:

        :tr: :td:foo:: :td:bar:: ::

        ::

        :caption:
        This is a *caption* with shortcuts $2+2$.

        ::
        """,
        want=r"""        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <tbody class="tablebody" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">foo</td>

        <td class="tabledatum" data-nodeid="6">bar</td>

        </tr>

        </tbody>

        <caption>
        <span class="label">Table 1. </span>This is a <span class="span" data-nodeid="10"><em>caption</em></span> with shortcuts <span class="math" data-nodeid="13">\(2+2\)</span>.
        </caption>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_reference_to_table():
    compare_have_want(
        have="""
        :table:{:label: lbl}

        :tbody:

        :tr: :td:foo:: :td:bar:: ::

        ::

        ::

        This :ref:lbl:: refers to the table.
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table id="lbl" class="table" data-nodeid="1">

        <tbody class="tablebody" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">foo</td>

        <td class="tabledatum" data-nodeid="6">bar</td>

        </tr>

        </tbody>

        </table>

        </div>

        <div class="paragraph" data-nodeid="8">

        <p>This <a class="reference" href="#lbl">Table 1</a> refers to the table.</p>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_reference_to_table_with_reftext():
    compare_have_want(
        have="""
        :table: {:label: lbl}

        :tbody:

        :tr: :td:foo:: :td:bar:: ::

        ::

        ::

        This :ref:lbl,refers:: to the table.
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table id="lbl" class="table" data-nodeid="1">

        <tbody class="tablebody" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">foo</td>

        <td class="tabledatum" data-nodeid="6">bar</td>

        </tr>

        </tbody>

        </table>

        </div>

        <div class="paragraph" data-nodeid="8">

        <p>This <a class="reference" href="#lbl">refers</a> to the table.</p>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_extra_space_within_tr():
    compare_have_want(
        have="""
        :table:

        :tbody:

        :tr:
          :td:foo::
          :td:bar::
        ::

        ::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <tbody class="tablebody" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">foo</td>

        <td class="tabledatum" data-nodeid="6">bar</td>

        </tr>

        </tbody>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_real_life_example():
    compare_have_want(
        have=r"""
        :table: {:label: lbl}

          :thead:
            :tr: :td: $H^{\tau}$ :: :td: $H^{*}$ :: :td: $H^{c}$ :: ::
          ::
          :tbody:
            :tr: :td: $0.0001$ :: :td: $0.0146$ :: :td: $0.0549$ :: ::
            :tr: :td: $0.1222$ :: :td: $0.0139$ :: :td: $0.0106$ :: ::

          ::

          :caption:
          Values of $\epsilon$ and $\delta$ for select subgraphs of Gr-Qc.

        ::
        """,
        want=r"""        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table id="lbl" class="table" data-nodeid="1">

        <thead class="tablehead" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4"><span class="math" data-nodeid="5">\(H^{\tau}\)</span></td>

        <td class="tabledatum" data-nodeid="7"><span class="math" data-nodeid="8">\(H^{*}\)</span></td>

        <td class="tabledatum" data-nodeid="10"><span class="math" data-nodeid="11">\(H^{c}\)</span></td>

        </tr>

        </thead>

        <tbody class="tablebody" data-nodeid="13">

        <tr class="tablerow" data-nodeid="14">

        <td class="tabledatum" data-nodeid="15"><span class="math" data-nodeid="16">\(0.0001\)</span></td>

        <td class="tabledatum" data-nodeid="18"><span class="math" data-nodeid="19">\(0.0146\)</span></td>

        <td class="tabledatum" data-nodeid="21"><span class="math" data-nodeid="22">\(0.0549\)</span></td>

        </tr>

        <tr class="tablerow" data-nodeid="24">

        <td class="tabledatum" data-nodeid="25"><span class="math" data-nodeid="26">\(0.1222\)</span></td>

        <td class="tabledatum" data-nodeid="28"><span class="math" data-nodeid="29">\(0.0139\)</span></td>

        <td class="tabledatum" data-nodeid="31"><span class="math" data-nodeid="32">\(0.0106\)</span></td>

        </tr>

        </tbody>

        <caption>
        <span class="label">Table 1. </span>Values of <span class="math" data-nodeid="36">\(\epsilon\)</span> and <span class="math" data-nodeid="39">\(\delta\)</span> for select subgraphs of Gr-Qc.
        </caption>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_real_life_example_different_spacing():
    compare_have_want(
        have=r"""
        :table:
        {  :label: lbl}

          :thead:
          :tr:
            :td: $H^{\tau}$ ::
            :td: $H^{*}$ ::
            :td: $H^{c}$ :: ::
          ::
          :tbody:
          :tr:
            :td: $0.0001$ ::
            :td: $0.0146$ ::
            :td: $0.0549$ :: ::
          :tr:
            :td: $0.1222$ ::
            :td: $0.0139$ ::
            :td: $0.0106$ ::
          ::
          ::

          :caption:
          Values of $\epsilon$ and $\delta$ for select subgraphs of Gr-Qc.

        ::
        """,
        want=r"""        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table id="lbl" class="table" data-nodeid="1">

        <thead class="tablehead" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4"><span class="math" data-nodeid="5">\(H^{\tau}\)</span></td>

        <td class="tabledatum" data-nodeid="7"><span class="math" data-nodeid="8">\(H^{*}\)</span></td>

        <td class="tabledatum" data-nodeid="10"><span class="math" data-nodeid="11">\(H^{c}\)</span></td>

        </tr>

        </thead>

        <tbody class="tablebody" data-nodeid="13">

        <tr class="tablerow" data-nodeid="14">

        <td class="tabledatum" data-nodeid="15"><span class="math" data-nodeid="16">\(0.0001\)</span></td>

        <td class="tabledatum" data-nodeid="18"><span class="math" data-nodeid="19">\(0.0146\)</span></td>

        <td class="tabledatum" data-nodeid="21"><span class="math" data-nodeid="22">\(0.0549\)</span></td>

        </tr>

        <tr class="tablerow" data-nodeid="24">

        <td class="tabledatum" data-nodeid="25"><span class="math" data-nodeid="26">\(0.1222\)</span></td>

        <td class="tabledatum" data-nodeid="28"><span class="math" data-nodeid="29">\(0.0139\)</span></td>

        <td class="tabledatum" data-nodeid="31"><span class="math" data-nodeid="32">\(0.0106\)</span></td>

        </tr>

        </tbody>

        <caption>
        <span class="label">Table 1. </span>Values of <span class="math" data-nodeid="36">\(\epsilon\)</span> and <span class="math" data-nodeid="39">\(\delta\)</span> for select subgraphs of Gr-Qc.
        </caption>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_tr_shortcut():
    compare_have_want(
        have="""
        :table:

        :thead:
        :tr: foo : bar : baz ::
        ::
        :tbody:
        :tr: a1 : a2 : a3 ::
        :tr: b1 : b2 : b3 ::
        ::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <thead class="tablehead" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">foo</td>

        <td class="tabledatum" data-nodeid="6">bar</td>

        <td class="tabledatum" data-nodeid="8">baz</td>

        </tr>

        </thead>

        <tbody class="tablebody" data-nodeid="10">

        <tr class="tablerow" data-nodeid="11">

        <td class="tabledatum" data-nodeid="12">a1</td>

        <td class="tabledatum" data-nodeid="14">a2</td>

        <td class="tabledatum" data-nodeid="16">a3</td>

        </tr>

        <tr class="tablerow" data-nodeid="18">

        <td class="tabledatum" data-nodeid="19">b1</td>

        <td class="tabledatum" data-nodeid="21">b2</td>

        <td class="tabledatum" data-nodeid="23">b3</td>

        </tr>

        </tbody>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_tr_shortcut_with_math():
    compare_have_want(
        have=r"""
        :table:

        :thead:
        :tr: $H^{\tau}$ : $H^{\top}$ : $H^{c}$ ::
        ::
        :tbody:
        :tr: $0.0001$ : $0.0146$ : $0.0549$ ::
        :tr: $0.1222$ : $0.0139$ : $0.0106$ ::
        ::

        ::
        """,
        want=r"""        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <thead class="tablehead" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4"><span class="math" data-nodeid="5">\(H^{\tau}\)</span></td>

        <td class="tabledatum" data-nodeid="7"><span class="math" data-nodeid="8">\(H^{\top}\)</span></td>

        <td class="tabledatum" data-nodeid="10"><span class="math" data-nodeid="11">\(H^{c}\)</span></td>

        </tr>

        </thead>

        <tbody class="tablebody" data-nodeid="13">

        <tr class="tablerow" data-nodeid="14">

        <td class="tabledatum" data-nodeid="15"><span class="math" data-nodeid="16">\(0.0001\)</span></td>

        <td class="tabledatum" data-nodeid="18"><span class="math" data-nodeid="19">\(0.0146\)</span></td>

        <td class="tabledatum" data-nodeid="21"><span class="math" data-nodeid="22">\(0.0549\)</span></td>

        </tr>

        <tr class="tablerow" data-nodeid="24">

        <td class="tabledatum" data-nodeid="25"><span class="math" data-nodeid="26">\(0.1222\)</span></td>

        <td class="tabledatum" data-nodeid="28"><span class="math" data-nodeid="29">\(0.0139\)</span></td>

        <td class="tabledatum" data-nodeid="31"><span class="math" data-nodeid="32">\(0.0106\)</span></td>

        </tr>

        </tbody>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_tr_shortcut_with_newlines():
    compare_have_want(
        have="""
        :table:

        :thead:
        :tr:
        foo : bar : baz
        ::
        ::
        :tbody:
        :tr:
        a1 : a2 : a3
        ::
        :tr:
        b1 : b2 : b3
        ::
        ::

        ::
        """,
        want="""        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <thead class="tablehead" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">foo</td>

        <td class="tabledatum" data-nodeid="6">bar</td>

        <td class="tabledatum" data-nodeid="8">baz</td>

        </tr>

        </thead>

        <tbody class="tablebody" data-nodeid="10">

        <tr class="tablerow" data-nodeid="11">

        <td class="tabledatum" data-nodeid="12">a1</td>

        <td class="tabledatum" data-nodeid="14">a2</td>

        <td class="tabledatum" data-nodeid="16">a3</td>

        </tr>

        <tr class="tablerow" data-nodeid="18">

        <td class="tabledatum" data-nodeid="19">b1</td>

        <td class="tabledatum" data-nodeid="21">b2</td>

        <td class="tabledatum" data-nodeid="23">b3</td>

        </tr>

        </tbody>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_tr_shortcut_with_escaped_colon():
    compare_have_want(
        have=r"""
        :table:

        :thead:
        :tr: foo : bar : baz ::
        ::
        :tbody:
        :tr: a\: : a2 : a3 ::
        :tr: b1  : b2 : b3 ::
        ::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="table-wrapper">

        <table class="table" data-nodeid="1">

        <thead class="tablehead" data-nodeid="2">

        <tr class="tablerow" data-nodeid="3">

        <td class="tabledatum" data-nodeid="4">foo</td>

        <td class="tabledatum" data-nodeid="6">bar</td>

        <td class="tabledatum" data-nodeid="8">baz</td>

        </tr>

        </thead>

        <tbody class="tablebody" data-nodeid="10">

        <tr class="tablerow" data-nodeid="11">

        <td class="tabledatum" data-nodeid="12">a:</td>

        <td class="tabledatum" data-nodeid="14">a2</td>

        <td class="tabledatum" data-nodeid="16">a3</td>

        </tr>

        <tr class="tablerow" data-nodeid="18">

        <td class="tabledatum" data-nodeid="19">b1</td>

        <td class="tabledatum" data-nodeid="21">b2</td>

        <td class="tabledatum" data-nodeid="23">b3</td>

        </tr>

        </tbody>

        </table>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )



"""All tags available in RSM markup."""

from collections import namedtuple
from datetime import datetime
from pathlib import Path

############
# Block tags
############
BlockTagInfo = namedtuple("BlockTagInfo", ["shorthand"])

abstract = BlockTagInfo(None)
"""Manuscript abstract."""
algorithm = BlockTagInfo(None)
"""Algorithm listing."""
appendix = BlockTagInfo(None)
"""Mark where the appendix sections begin."""
author = BlockTagInfo(None)
"""Author information."""
references = BlockTagInfo(None)
"""List of bibliography items."""
codeblock = BlockTagInfo(None)
"""Code listing."""
corollary = BlockTagInfo(None)
"""Corollary."""
definition = BlockTagInfo(None)
"""Definition."""
enumerate = BlockTagInfo(None)  # pylint: disable=redefined-builtin
"""Numbered list."""
example = BlockTagInfo(None)
"""Example."""
figure = BlockTagInfo(None)
"""Figure float."""
html = BlockTagInfo(None)
"""HTML asset."""
itemize = BlockTagInfo(None)
"""Unnumbered list."""
lemma = BlockTagInfo(None)
"""Lemma."""
mathblock = BlockTagInfo(None)
"""Math block (a.k.a. display math)."""
proof = BlockTagInfo(None)
"""Proof of Theorem."""
proposition = BlockTagInfo(None)
"""Proposition."""
remark = BlockTagInfo(None)
"""Remark."""
section = BlockTagInfo(None)
"""Section."""
sketch = BlockTagInfo(None)
"""Proof sketch."""
subsection = BlockTagInfo(None)
"""Subsection."""
subsubsection = BlockTagInfo(None)
"""Sub-subsection."""
theorem = BlockTagInfo(None)
"""Theorem."""
toc = BlockTagInfo(None)
"""Table of Contents"""
video = BlockTagInfo(None)
"""Video asset."""


################
# Paragraph tags
################
ParagraphTagInfo = namedtuple("ParagraphTagInfo", ["x"])

item = ParagraphTagInfo(None)
"""Item in enumerate or itemize."""
caption = ParagraphTagInfo(None)
"""Figure or table caption."""
paragraph = ParagraphTagInfo(None)
"""Generic paragraph.  Use when adding meta tags to a paragraph."""


#############
# Inline tags
#############
InlineTagInfo = namedtuple("InlineTagInfo", ["shorthand"])

code = InlineTagInfo(None)
"""Inline code."""
cite = InlineTagInfo(None)
"""Citation."""
claim = InlineTagInfo(None)
"""Mathematical claim."""
draft = InlineTagInfo(None)
"""A visible comment."""
math = InlineTagInfo(None)
"""Inline math."""
note = InlineTagInfo(None)
"""Footnote."""
ref = InlineTagInfo(None)
"""Link to internal label."""
span = InlineTagInfo(None)
"""Text span."""
url = InlineTagInfo(None)
"""Link to external URL."""


###########
# Meta tags
###########


MetaTagInfo = namedtuple("MetaTagInfo", ["parent", "type"])

affiliation = MetaTagInfo(None, datetime)
"""Author institutional affiliation."""
author_display_first = MetaTagInfo(None, int)
"""Number of authors to display at start when >5 authors."""
author_display_last = MetaTagInfo(None, int)
"""Number of authors to display at end when >5 authors."""
author_note = MetaTagInfo(None, str)
"""Author note (e.g., 'Equal contribution')."""
date = MetaTagInfo(None, datetime)
"""Manuscript date."""
email = MetaTagInfo(None, str)
"""Author email."""
emphas = MetaTagInfo(None, bool)
"""Whether span is emphasized."""
goal = MetaTagInfo(None, "?")
"""Theorem goal."""
icon = MetaTagInfo(None, list)
"""Icon assigned to the block."""
isclaim = MetaTagInfo(None, bool)
"""Whether a math block is a claim."""
keywords = MetaTagInfo(None, list)
"""Abstract keywords."""
label = MetaTagInfo(None, str)
"""Tag label."""
lang = MetaTagInfo(None, str)
"""Specify language in code and code blocks."""
msc = MetaTagInfo(None, list)
"""Mathematics Subject Classification."""
name = MetaTagInfo(None, str)
"""Author name."""
nonum = MetaTagInfo(None, bool)
"""Whether to number the tag."""
orcid = MetaTagInfo(None, str)
"""Author ORCID identifier."""
path = MetaTagInfo(None, Path)
"""Figure path."""
reftext = MetaTagInfo(None, str)
"""Label reftext."""
scale = MetaTagInfo(None, str)
"""Figure scale."""
strong = MetaTagInfo(None, bool)
"""Whether span is strong."""
title = MetaTagInfo(None, str)
"""Manuscript or section title."""
types = MetaTagInfo(None, list)
"""Tag types."""


###########
# Math tags
###########
MathTagInfo = namedtuple("MathTagInfo", ["shorthand"])

assume = MathTagInfo(None)
"""Introduce an assumption."""
case = MathTagInfo(None)
"""Prove special case of the goal."""
define = MathTagInfo(None)
"""Introduce a variable and assumption."""
exercise = MathTagInfo(None)
"""Introduce an exercise."""
let = MathTagInfo(None)
"""Introduce a variable and assumption."""
new = MathTagInfo(None)
"""Introduce a variable and assumption."""
p = MathTagInfo(None)
"""Sub-proof."""
pick = MathTagInfo(None)
"""Introduce a variable with a specific property (see 'st')."""
porism = MathTagInfo(None)
"""Corollary of a proof."""
prev = MathTagInfo(None)
"""Reference the previous step."""
prev2 = MathTagInfo(None)
"""Reference the step before last."""
prev3 = MathTagInfo(None)
"""Reference the step two steps before."""
previous = MathTagInfo(None)
"""Reference the previous step."""
problem = MathTagInfo(None)
"""Introduce a problem."""
prove = MathTagInfo(None)
"""Set goal."""
qed = MathTagInfo(None)
"""State the goal is proven."""
st = MathTagInfo(None)
"""'Such that', part of 'pick'."""
step = MathTagInfo(None)
"""Proof step."""
suffices = MathTagInfo(None)
"""Change goal."""
suppose = MathTagInfo(None)
"""Introduce an assumption."""
then = MathTagInfo(None)
"""State a claim under an assumption."""
wlog = MathTagInfo(None)
"""Without loss of generality (introduce an assupmtion)."""
write = MathTagInfo(None)
"""Introduce a symbol."""


############
# Table tags
############
TableTagInfo = namedtuple("TableTagInfo", ["shorthand"])

table = TableTagInfo(None)
"""Table."""
tbody = TableTagInfo(None)
"""Table body."""
td = TableTagInfo(None)
"""Table datum."""
thead = TableTagInfo(None)
"""Table head."""
tr = TableTagInfo(None)
"""Table row."""



"""Tests for rsm CLI subcommand structure.

Tests that the rsm command works with subcommands: build, render, lint, serve.
"""

import subprocess

import pytest


class TestCLISubcommands:
    """Test that rsm command works with subcommands."""

    @pytest.mark.slow
    def test_rsm_build_subcommand(self, tmp_path):
        """Test that 'rsm build' works as a subcommand."""
        src = "# Test\n\nBuild subcommand.\n"
        src_file = tmp_path / "test.rsm"
        src_file.write_text(src)

        result = subprocess.run(
            f"rsm build {src_file}",
            cwd=tmp_path,
            shell=True,
            capture_output=True,
            check=True,
        )

        # Should create test.html (derived from test.rsm)
        output_html = tmp_path / "test.html"
        assert output_html.exists(), "rsm build should create test.html"
        assert "Build subcommand" in output_html.read_text()

    @pytest.mark.slow
    def test_rsm_render_subcommand(self, tmp_path):
        """Test that 'rsm render' works as a subcommand."""
        src = "# Test\n\nRender subcommand.\n"
        src_file = tmp_path / "test.rsm"
        src_file.write_text(src)

        result = subprocess.run(
            f"rsm render {src_file}",
            cwd=tmp_path,
            shell=True,
            capture_output=True,
            check=True,
        )

        # Should print HTML body to stdout (not full document)
        stdout = result.stdout.decode("utf-8")
        assert "<body>" in stdout
        assert "Render subcommand" in stdout

        # Should NOT create files
        assert not (tmp_path / "index.html").exists()

    @pytest.mark.slow
    def test_rsm_check_subcommand(self, tmp_path):
        """Test that 'rsm check' works as a subcommand."""
        src = "# Test\n\nCheck subcommand.\n"
        src_file = tmp_path / "test.rsm"
        src_file.write_text(src)

        result = subprocess.run(
            f"rsm check {src_file}",
            cwd=tmp_path,
            shell=True,
            capture_output=True,
            check=True,
        )

        # Should succeed without errors
        assert result.returncode == 0

    @pytest.mark.slow
    def test_rsm_build_no_serve_flag(self, tmp_path):
        """Test that 'rsm build --serve' is no longer valid."""
        src = "# Test\n\nNo serve flag.\n"
        src_file = tmp_path / "test.rsm"
        src_file.write_text(src)

        result = subprocess.run(
            f"rsm build {src_file} --serve",
            cwd=tmp_path,
            shell=True,
            capture_output=True,
        )

        # Should fail with error about unrecognized argument
        assert result.returncode != 0
        stderr = result.stderr.decode("utf-8")
        assert "--serve" in stderr or "unrecognized" in stderr.lower()

    @pytest.mark.slow
    def test_rsm_serve_subcommand_basic(self, tmp_path):
        """Test that 'rsm serve' subcommand exists and accepts arguments."""
        src = "# Test\n\nServe subcommand.\n"
        src_file = tmp_path / "test.rsm"
        src_file.write_text(src)

        # Just test that the command is recognized (we won't actually start the server)
        # We can test this by checking --help
        result = subprocess.run(
            "rsm serve --help",
            shell=True,
            capture_output=True,
        )

        # Should succeed and show help for serve subcommand
        assert result.returncode == 0
        stdout = result.stdout.decode("utf-8")
        assert "serve" in stdout.lower()

    @pytest.mark.slow
    def test_rsm_version_flag(self, tmp_path):
        """Test that 'rsm --version' works."""
        result = subprocess.run(
            "rsm --version",
            shell=True,
            capture_output=True,
        )

        # Should succeed and show version
        assert result.returncode == 0
        stdout = result.stdout.decode("utf-8")
        assert "rsm-markup" in stdout

    @pytest.mark.slow
    def test_rsm_help_shows_subcommands(self, tmp_path):
        """Test that 'rsm --help' shows available subcommands."""
        result = subprocess.run(
            "rsm --help",
            shell=True,
            capture_output=True,
        )

        # Should succeed and list subcommands
        assert result.returncode == 0
        stdout = result.stdout.decode("utf-8")
        assert "build" in stdout
        assert "render" in stdout
        assert "check" in stdout
        assert "serve" in stdout

    @pytest.mark.slow
    def test_rsm_build_with_output_flag(self, tmp_path):
        """Test that 'rsm build -o' flag still works."""
        src = "# Test\n\nOutput flag.\n"
        src_file = tmp_path / "test.rsm"
        src_file.write_text(src)

        subprocess.run(
            f"rsm build {src_file} -o custom",
            cwd=tmp_path,
            shell=True,
            capture_output=True,
            check=True,
        )

        # Should create custom.html
        output_html = tmp_path / "custom.html"
        assert output_html.exists()
        assert "Output flag" in output_html.read_text()

    @pytest.mark.slow
    def test_rsm_build_with_standalone_flag(self, tmp_path):
        """Test that 'rsm build --standalone' flag still works."""
        src = "# Test\n\nStandalone.\n"
        src_file = tmp_path / "test.rsm"
        src_file.write_text(src)

        subprocess.run(
            f"rsm build {src_file} --standalone",
            cwd=tmp_path,
            shell=True,
            capture_output=True,
            check=True,
        )

        # Should create test.html without static/ folder
        output_html = tmp_path / "test.html"
        assert output_html.exists()
        assert not (tmp_path / "static").exists()



from conftest import compare_have_want_handrails


def test_two_steps():
    compare_have_want_handrails(
        have="""
        # Some Title

        :proof:

          :step: Foo.::

          :step: Bar.::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>Some Title</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="proof hr hr-labeled" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Proof</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item collapse-steps">
            <span class="icon collapse-all">
            </span>
            <span class="hr-menu-item-text">Collapse all</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="paragraph hr-label">

        <p><span class="span label">Proof.</span></p>

        </div>

        <div class="step hr hr-offset" tabindex=0 data-nodeid="2">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Step ⟨1⟩</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item collapse-subproof disabled">
            <span class="icon collapse">
            </span>
            <span class="hr-menu-item-text">Collapse</span>
          </div>

          <div class="hr-menu-item collapse-steps disabled">
            <span class="icon collapse-all">
            </span>
            <span class="hr-menu-item-text">Collapse all</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="statement" data-nodeid="3">

        <div class="paragraph hr hr-hidden hr-offset" tabindex=0 data-nodeid="4">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Foo.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"><div class="step-number"><p>⟨1⟩</p></div></div>
        </div>

        </div>

        <div class="step last hr hr-offset" tabindex=0 data-nodeid="6">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Step ⟨2⟩</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item collapse-subproof disabled">
            <span class="icon collapse">
            </span>
            <span class="hr-menu-item-text">Collapse</span>
          </div>

          <div class="hr-menu-item collapse-steps disabled">
            <span class="icon collapse-all">
            </span>
            <span class="hr-menu-item-text">Collapse all</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="statement" data-nodeid="7">

        <div class="paragraph hr hr-hidden hr-offset" tabindex=0 data-nodeid="8">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Bar.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"><div class="step-number"><p>⟨2⟩</p></div></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_sub_step():
    compare_have_want_handrails(
        have="""
        # Some Title

        :proof:

          :step: Top level step.

            :step: Sub-step.

              :p: Sub-Proof.::

            ::
          ::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>Some Title</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="proof hr hr-labeled" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Proof</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item collapse-steps">
            <span class="icon collapse-all">
            </span>
            <span class="hr-menu-item-text">Collapse all</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="paragraph hr-label">

        <p><span class="span label">Proof.</span></p>

        </div>

        <div class="step last hr hr-offset" tabindex=0 data-nodeid="2">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Step ⟨1⟩</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item collapse-subproof">
            <span class="icon collapse">
            </span>
            <span class="hr-menu-item-text">Collapse</span>
          </div>

          <div class="hr-menu-item collapse-steps">
            <span class="icon collapse-all">
            </span>
            <span class="hr-menu-item-text">Collapse all</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="statement" data-nodeid="3">

        <div class="paragraph hr hr-hidden hr-offset" tabindex=0 data-nodeid="4">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Top level step.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="subproof hr hr-offset hr-shift-1" data-nodeid="6">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        </div>

        <div class="hr-border-zone">
        <div class="hr-border-rect"></div>
        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="step hr hr-hidden hr-offset" tabindex=0 data-nodeid="7">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Step ⟨1.1⟩</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item collapse-subproof">
            <span class="icon collapse">
            </span>
            <span class="hr-menu-item-text">Collapse</span>
          </div>

          <div class="hr-menu-item collapse-steps disabled">
            <span class="icon collapse-all">
            </span>
            <span class="hr-menu-item-text">Collapse all</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="statement" data-nodeid="8">

        <div class="paragraph hr hr-offset hr-hidden" tabindex=0 data-nodeid="9">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Sub-step.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="subproof hr hr-offset hr-shift-1" data-nodeid="11">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        </div>

        <div class="hr-border-zone">
        <div class="hr-border-rect"></div>
        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="paragraph hr hr-offset hr-hidden" tabindex=0 data-nodeid="12">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Sub-Proof.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"><div class="step-number"><p>⟨1.1⟩</p></div></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"><div class="step-number"><p>⟨1⟩</p></div></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_theorem():
    compare_have_want_handrails(
        have="""
        :theorem:

        Hello.

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="theorem hr hr-labeled" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Theorem 1</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="paragraph hr-label">

        <p><span class="span label">Theorem 1.</span></p>

        </div>

        <div class="paragraph hr hr-offset hr-hidden" tabindex=0 data-nodeid="2">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Hello.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_proof():
    compare_have_want_handrails(
        have="""
        # Some Title

        :proof:

          :step: Bar.::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>Some Title</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="proof hr hr-labeled" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Proof</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item collapse-steps">
            <span class="icon collapse-all">
            </span>
            <span class="hr-menu-item-text">Collapse all</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="paragraph hr-label">

        <p><span class="span label">Proof.</span></p>

        </div>

        <div class="step last hr hr-offset" tabindex=0 data-nodeid="2">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Step ⟨1⟩</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item collapse-subproof disabled">
            <span class="icon collapse">
            </span>
            <span class="hr-menu-item-text">Collapse</span>
          </div>

          <div class="hr-menu-item collapse-steps disabled">
            <span class="icon collapse-all">
            </span>
            <span class="hr-menu-item-text">Collapse all</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="statement" data-nodeid="3">

        <div class="paragraph hr hr-hidden hr-offset" tabindex=0 data-nodeid="4">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Bar.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"><div class="step-number"><p>⟨1⟩</p></div></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_proof_with_sketch():
    compare_have_want_handrails(
        have="""
        # Some Title

        :sketch: Foo.::

        :proof:

          :step: Bar.::

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>Some Title</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="sketch hr hr-labeled" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Sketch</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="paragraph hr-label">

        <p><span class="span label">Proof sketch.</span></p>

        </div>

        <div class="paragraph hr hr-offset hr-hidden" tabindex=0 data-nodeid="2">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Foo.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="proof hr hr-labeled" tabindex=0 data-nodeid="4">

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Proof</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item collapse-steps">
            <span class="icon collapse-all">
            </span>
            <span class="hr-menu-item-text">Collapse all</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="paragraph hr-label">

        <p><span class="span label">Proof.</span></p>

        </div>

        <div class="step last hr hr-offset" tabindex=0 data-nodeid="5">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Step ⟨1⟩</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item collapse-subproof disabled">
            <span class="icon collapse">
            </span>
            <span class="hr-menu-item-text">Collapse</span>
          </div>

          <div class="hr-menu-item collapse-steps disabled">
            <span class="icon collapse-all">
            </span>
            <span class="hr-menu-item-text">Collapse all</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="statement" data-nodeid="6">

        <div class="paragraph hr hr-hidden hr-offset" tabindex=0 data-nodeid="7">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Bar.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"><div class="step-number"><p>⟨1⟩</p></div></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )



from rsm import translator as tr


def test_edit_after_creation():
    cmd = tr.AppendOpenCloseTag(
        content="some content",
        classes=["foo"],
        newline_inner=False,
        newline_outer=False,
    )
    assert cmd.make_text() == '<div class="foo">some content</div>'

    cmd.classes.append("bar")
    assert cmd.make_text() == '<div class="foo bar">some content</div>'



from conftest import compare_have_want_handrails


def test_codeblock_handrails():
    compare_have_want_handrails(
        have="""\
        ```
        comp = [abs(x) for x in range(10)]
        ```
        """,
        want="""\
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="codeblock hr hr-hidden" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Code Block</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <pre>
        <code>comp = [abs(x) for x in range(10)]</code>
        </pre>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_codeblock_with_lang_handrails():
    compare_have_want_handrails(
        have="""\
        ```{:lang: python}

        def hello():
            print("world")
        ```
        """,
        want="""\
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="codeblock hr hr-hidden" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Code Block</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <pre>
        <code class="highlight python"><span class="k">def</span><span class="w"> </span><span class="nf">hello</span><span class="p">():</span>
            <span class="nb">print</span><span class="p">(</span><span class="s2">&quot;world&quot;</span><span class="p">)</span>
        </code>
        </pre>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_manuscript():
    compare_have_want_handrails(
        have="""
        # Some Title

        Hello.

        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>Some Title</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Hello.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_section():
    compare_have_want_handrails(
        have="""
        # Some Title

        ## Section
        Hello.

        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>Some Title</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <section class="section level-2" data-nodeid="1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Section 1</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h2>1. Section</h2>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="2">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Hello.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_abstract():
    compare_have_want_handrails(
        have="""
        # Some Title

        :abstract:
          The abstract.
        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>Some Title</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="abstract" data-nodeid="1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Abstract</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h3>Abstract</h3>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="2">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>The abstract.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_theorem():
    compare_have_want_handrails(
        have="""
        :theorem:

        Hello.

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="theorem hr hr-labeled" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Theorem 1</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <div class="paragraph hr-label">

        <p><span class="span label">Theorem 1.</span></p>

        </div>

        <div class="paragraph hr hr-offset hr-hidden" tabindex=0 data-nodeid="2">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Hello.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_author():
    compare_have_want_handrails(
        have=r"""
        # Indefinite Linear Algebra of the NBM {
          :date: 2024-04-13
        }

        :author: {
          :name: Leo Torres
          :email: leo@leotrs.com
        }
        ::
        """,
        want=r"""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>Indefinite Linear Algebra of the NBM</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <p class="manuscript-date">April 13, 2024</p>

        <div class="author hr hr-hidden" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Author</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Leo Torres</p>

        <p><a href="mailto:leo@leotrs.com">leo@leotrs.com</a></p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_bibliography():
    compare_have_want_handrails(
        have="""
        This is a citation :cite:atiyah2018introduction::.

        :references:

        @book{atiyah2018introduction,
          title={Introduction to commutative algebra},
          author={Atiyah, M.F., & MacDonald, I.G.},
          year={2018},
          publisher={CRC Press},
          doi={https://doi.org/10.1201/9780429493638},
        }

        ::
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>This is a citation [<a id="cite-0" class="reference cite" href="#atiyah2018introduction">1</a>].</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <section class="level-2">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Bibliography</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h2>References</h2>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="bibliography" data-nodeid="5">

        <div id="atiyah2018introduction" class="bibitem" data-nodeid="6">

        <div class="hr-hidden hr" tabindex=0>

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">
        <p>1. Atiyah, M.F., & MacDonald, I.G. "Introduction to commutative algebra". CRC Press. 2018. <br />[<a class="reference backlink" href="#cite-0">↖1</a>]</p>
        </div>

        <div class="hr-info-zone">
        <div class="hr-info">
        <a id="atiyah2018introduction-doi" class="bibitem-doi" href="https://doi.org/https://doi.org/10.1201/9780429493638" target="_blank">
        <div class="icon ext">
                        </div></a>
        </div>
        </div>

        </div>

        </div>

        </div>

        </section>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_inline_math_followed_by_dot():
    compare_have_want_handrails(
        have="""
        # title

        one $2+2=4$.

        two $2+2=4$ baz.

        three $2+2=4$. Another sentence.
        """,
        want=r"""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>title</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>one <span class="inline-math-wrapper">
        <span class="math" data-nodeid="3">\(2+2=4\)</span><span>.</span></span></p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="6">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>two <span class="math" data-nodeid="8">\(2+2=4\)</span> baz.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="11">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>three <span class="inline-math-wrapper">
        <span class="math" data-nodeid="13">\(2+2=4\)</span><span>.</span></span> Another sentence.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_mathblock_nonum():
    compare_have_want_handrails(
        have="""
        This one has a number
        $$
        2+2=4
        $$

        And this one does not
        $$
        {:nonum:}
        2+2=4
        $$
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="1">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>This one has a number </p>
        <div class="mathblock hr hr-hidden hr-offset" tabindex=0 data-nodeid="3">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Equation (1)</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">
        $$
        2+2=4
        $$
        </div>

        <div class="hr-info-zone">
        <div class="hr-info"><div class="eqn-number"><p>(1)</p></div></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="5">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>And this one does not </p>
        <div class="mathblock hr hr-hidden hr-offset" tabindex=0 data-nodeid="7">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Equation</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">
        $$
        2+2=4
        $$
        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_toc_no_labels():
    compare_have_want_handrails(
        have="""
        # Foo

        :abstract:
          Abs.
        ::

        :toc:

        ## Section

        ## Section

        ### Sub-section
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">
        <div class="float-minimap-wrapper">
        <div class="minimap">

                <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 32 104" fill="#3C4952" preserveAspectRatio="xMinYMid meet" stroke-width="0">
                  <defs>

                    <linearGradient id="purple-green-scroll" x1="0%" x2="0%" y1="0%" y2="100%" gradientUnits="userSpaceOnUse">
                      <stop offset="0%" stop-color="#AD71F2" />
                      <stop id="stop-follow-scroll-1" offset="100%" stop-color="#1FB5A2" />
                      <stop id="stop-follow-scroll-2" offset="100%" stop-color="#DAE1E5" />
                      <stop offset="100%" stop-color="#DAE1E5" />
                    </linearGradient>

                    <mask id="gradient-mask">
                      <rect width="100%" height="100%" fill="url(#purple-green-scroll)" />
                    </mask>

                  </defs>

                  <g fill="url(#purple-green-scroll)">
                    <rect x="14" y="24" width="4" height="56" />
            <a href="#" class="reference" tabindex="-1"><circle id="mm-" cx="16" cy="20" r="8" data-pos="20" />
            <circle cx="16" cy="20" r="4" fill="#FCFEFF" data-pos="20" /></a>
            <a href="#" class="reference" tabindex="-1"><circle id="mm-" cx="16" cy="52" r="8" data-pos="52" />
            <circle cx="16" cy="52" r="4" fill="#FCFEFF" data-pos="52" /></a>
            <a href="#" class="reference" tabindex="-1"><circle id="mm-" cx="16" cy="84" r="6" data-pos="84" />
            <circle cx="16" cy="84" r="2" fill="#FCFEFF" data-pos="84" /></a></g>
        </svg>
        </div>
        </div>
        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>Foo</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="abstract" data-nodeid="1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Abstract</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h3>Abstract</h3>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="2">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Abs.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="toc">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Contents</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h3>Table of Contents</h3>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="toc-wrapper">

        <div class="minimap">

                <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 32 104" fill="#3C4952" preserveAspectRatio="xMinYMid meet" stroke-width="0">
                  <defs>

                    <linearGradient id="purple-green-mouse" x1="0%" x2="0%" y1="0%" y2="100%" gradientUnits="userSpaceOnUse">
                      <stop offset="0%" stop-color="#AD71F2" />
                      <stop id="stop-follow-mouse-1" offset="100%" stop-color="#1FB5A2" />
                      <stop id="stop-follow-mouse-2" offset="100%" stop-color="#DAE1E5" />
                      <stop offset="100%" stop-color="#DAE1E5" />
                    </linearGradient>

                    <mask id="gradient-mask">
                      <rect width="100%" height="100%" fill="url(#purple-green-mouse)" />
                    </mask>

                  </defs>

                  <g fill="url(#purple-green-mouse)">
                    <rect x="14" y="24" width="4" height="56" />
            <circle cx="16" cy="20" r="8" data-pos="20" />
            <circle cx="16" cy="20" r="4" fill="#FCFEFF" data-pos="20" />
            <circle cx="16" cy="52" r="8" data-pos="52" />
            <circle cx="16" cy="52" r="4" fill="#FCFEFF" data-pos="52" />
            <circle cx="16" cy="84" r="6" data-pos="84" />
            <circle cx="16" cy="84" r="2" fill="#FCFEFF" data-pos="84" /></g>
        </svg>
        </div>

        <ul class="contents" data-nodeid="4">

        <li class="item" data-nodeid="5">
        <a class="reference" href="#">1. Section</a>
        </li>

        <li class="item" data-nodeid="7">
        <a class="reference" href="#">2. Section</a>
        <ul class="itemize" data-nodeid="9">

        <li class="item" data-nodeid="10">
        <a class="reference" href="#">2.1. Sub-section</a>
        </li>

        </ul>

        </li>

        </ul>

        </div>

        </div>

        <section class="section level-2" data-nodeid="12">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Section 1</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h2>1. Section</h2>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        <section class="section level-2" data-nodeid="13">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Section 2</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h2>2. Section</h2>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <section class="subsection level-3" data-nodeid="14">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Section 2.1</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h3>2.1. Sub-section</h3>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </section>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_toc_with_labels():
    compare_have_want_handrails(
        have="""
        # Foo

        :abstract:
          Abs.
        ::

        :toc:

        ## Section
          {:label: sec-1}

        ## Section
          {:label: sec-2}

        ### Sub-section
          {:label: sub-sec}
        """,
        want="""
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">
        <div class="float-minimap-wrapper">
        <div class="minimap">

                <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 32 104" fill="#3C4952" preserveAspectRatio="xMinYMid meet" stroke-width="0">
                  <defs>

                    <linearGradient id="purple-green-scroll" x1="0%" x2="0%" y1="0%" y2="100%" gradientUnits="userSpaceOnUse">
                      <stop offset="0%" stop-color="#AD71F2" />
                      <stop id="stop-follow-scroll-1" offset="100%" stop-color="#1FB5A2" />
                      <stop id="stop-follow-scroll-2" offset="100%" stop-color="#DAE1E5" />
                      <stop offset="100%" stop-color="#DAE1E5" />
                    </linearGradient>

                    <mask id="gradient-mask">
                      <rect width="100%" height="100%" fill="url(#purple-green-scroll)" />
                    </mask>

                  </defs>

                  <g fill="url(#purple-green-scroll)">
                    <rect x="14" y="24" width="4" height="56" />
            <a href="#sec-1" class="reference" tabindex="-1"><circle id="mm-sec-1" cx="16" cy="20" r="8" data-pos="20" />
            <circle cx="16" cy="20" r="4" fill="#FCFEFF" data-pos="20" /></a>
            <a href="#sec-2" class="reference" tabindex="-1"><circle id="mm-sec-2" cx="16" cy="52" r="8" data-pos="52" />
            <circle cx="16" cy="52" r="4" fill="#FCFEFF" data-pos="52" /></a>
            <a href="#sub-sec" class="reference" tabindex="-1"><circle id="mm-sub-sec" cx="16" cy="84" r="6" data-pos="84" />
            <circle cx="16" cy="84" r="2" fill="#FCFEFF" data-pos="84" /></a></g>
        </svg>
        </div>
        </div>
        <section class="level-1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Title</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h1>Foo</h1>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="abstract" data-nodeid="1">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Abstract</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h3>Abstract</h3>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="paragraph hr hr-hidden" tabindex=0 data-nodeid="2">

        <div class="hr-collapse-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Paragraph</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <p>Abs.</p>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </div>

        <div class="toc">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Contents</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link disabled">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h3>Table of Contents</h3>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <div class="toc-wrapper">

        <div class="minimap">

                <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 32 104" fill="#3C4952" preserveAspectRatio="xMinYMid meet" stroke-width="0">
                  <defs>

                    <linearGradient id="purple-green-mouse" x1="0%" x2="0%" y1="0%" y2="100%" gradientUnits="userSpaceOnUse">
                      <stop offset="0%" stop-color="#AD71F2" />
                      <stop id="stop-follow-mouse-1" offset="100%" stop-color="#1FB5A2" />
                      <stop id="stop-follow-mouse-2" offset="100%" stop-color="#DAE1E5" />
                      <stop offset="100%" stop-color="#DAE1E5" />
                    </linearGradient>

                    <mask id="gradient-mask">
                      <rect width="100%" height="100%" fill="url(#purple-green-mouse)" />
                    </mask>

                  </defs>

                  <g fill="url(#purple-green-mouse)">
                    <rect x="14" y="24" width="4" height="56" />
            <circle cx="16" cy="20" r="8" data-pos="20" />
            <circle cx="16" cy="20" r="4" fill="#FCFEFF" data-pos="20" />
            <circle cx="16" cy="52" r="8" data-pos="52" />
            <circle cx="16" cy="52" r="4" fill="#FCFEFF" data-pos="52" />
            <circle cx="16" cy="84" r="6" data-pos="84" />
            <circle cx="16" cy="84" r="2" fill="#FCFEFF" data-pos="84" /></g>
        </svg>
        </div>

        <ul class="contents" data-nodeid="4">

        <li class="item" data-nodeid="5">
        <a class="reference" href="#sec-1">1. Section</a>
        </li>

        <li class="item" data-nodeid="7">
        <a class="reference" href="#sec-2">2. Section</a>
        <ul class="itemize" data-nodeid="9">

        <li class="item" data-nodeid="10">
        <a class="reference" href="#sub-sec">2.1. Sub-section</a>
        </li>

        </ul>

        </li>

        </ul>

        </div>

        </div>

        <section id="sec-1" class="section level-2" data-nodeid="12">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Section 1</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h2>1. Section</h2>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        <section id="sec-2" class="section level-2" data-nodeid="13">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Section 2</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h2>2. Section</h2>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        <section id="sub-sec" class="subsection level-3" data-nodeid="14">

        <div class="heading hr" tabindex=0>

        <div class="hr-collapse-zone">

                    <div class="hr-collapse">
                      <div class="icon collapse">
                      </div>
                    </div>

        </div>

        <div class="hr-menu-zone">

        <div class="hr-menu">

          <div class="hr-menu-label">
            <span class="hr-menu-item-text">Section 2.1</span>
          </div>

          <div class="hr-menu-separator"></div>

          <div class="hr-menu-item link">
            <span class="icon link">
            </span>
            <span class="hr-menu-item-text">Copy link</span>
          </div>

          <div class="hr-menu-item">
            <span class="icon code">
            </span>
            <span class="hr-menu-item-text">Source</span>
          </div>

        </div>

        </div>

        <div class="hr-border-zone">

                        <div class="hr-border-dots">
                          <div class="icon dots">
                          </div>
                        </div>
                        <div class="hr-border-rect">
                        </div>

        </div>

        <div class="hr-spacer-zone">
        <div class="hr-spacer"></div>
        </div>

        <div class="hr-content-zone">

        <h3>2.1. Sub-section</h3>

        </div>

        <div class="hr-info-zone">
        <div class="hr-info"></div>
        </div>

        </div>

        </section>

        </section>

        </section>

        </div>

        </div>

        </body>
        """,
    )



import pytest

import rsm
from conftest import compare_have_want


def test_works_with_no_reftext_and_label():
    compare_have_want(
        have="""\
        :proof:

          :step:
            {:label: lbl}
          Foo.
          ::

          :step: Bar :previous:1::.::

        ::
        """,
        want="""\
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="proof" data-nodeid="1">

        <div class="paragraph hr-label">

        <p><span class="span label">Proof.</span></p>

        </div>

        <div id="lbl" class="step" data-nodeid="2">

        <div class="statement" data-nodeid="3">

        <div class="paragraph" data-nodeid="4">

        <p>Foo.</p>

        </div>

        </div>

        </div>

        <div class="step last" data-nodeid="6">

        <div class="statement" data-nodeid="7">

        <div class="paragraph" data-nodeid="8">

        <p>Bar <a class="reference" href="#lbl">Step ⟨1⟩</a>.</p>

        </div>

        </div>

        </div>

        <div class="halmos"></div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_works_with_no_reftext_and_no_label():
    compare_have_want(
        have="""\
        :proof:

          :step: Foo.::

          :step: Bar :previous:1::.::

        ::
        """,
        want="""\
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="proof" data-nodeid="1">

        <div class="paragraph hr-label">

        <p><span class="span label">Proof.</span></p>

        </div>

        <div class="step" data-nodeid="2">

        <div class="statement" data-nodeid="3">

        <div class="paragraph" data-nodeid="4">

        <p>Foo.</p>

        </div>

        </div>

        </div>

        <div class="step last" data-nodeid="6">

        <div class="statement" data-nodeid="7">

        <div class="paragraph" data-nodeid="8">

        <p>Bar <a class="reference" href="#">Step ⟨1⟩</a>.</p>

        </div>

        </div>

        </div>

        <div class="halmos"></div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_works_with_reftext_and_label():
    compare_have_want(
        have="""\
        :proof:

          :step:
            {:label: lbl}
          Foo.::

          :step: Bar :previous:1,bar::.::

        ::
        """,
        want="""\
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="proof" data-nodeid="1">

        <div class="paragraph hr-label">

        <p><span class="span label">Proof.</span></p>

        </div>

        <div id="lbl" class="step" data-nodeid="2">

        <div class="statement" data-nodeid="3">

        <div class="paragraph" data-nodeid="4">

        <p>Foo.</p>

        </div>

        </div>

        </div>

        <div class="step last" data-nodeid="6">

        <div class="statement" data-nodeid="7">

        <div class="paragraph" data-nodeid="8">

        <p>Bar <a class="reference" href="#lbl">bar</a>.</p>

        </div>

        </div>

        </div>

        <div class="halmos"></div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )


def test_works_with_reftext_and_no_label(caplog):
    compare_have_want(
        have="""\
        :proof:

          :step: Foo.::

          :step: Bar :previous:1,bar::.::

        ::
        """,
        want="""\
        <body>

        <div class="manuscriptwrapper">

        <div class="manuscript" data-nodeid="0">

        <section class="level-1">

        <div class="proof" data-nodeid="1">

        <div class="paragraph hr-label">

        <p><span class="span label">Proof.</span></p>

        </div>

        <div class="step" data-nodeid="2">

        <div class="statement" data-nodeid="3">

        <div class="paragraph" data-nodeid="4">

        <p>Foo.</p>

        </div>

        </div>

        </div>

        <div class="step last" data-nodeid="6">

        <div class="statement" data-nodeid="7">

        <div class="paragraph" data-nodeid="8">

        <p>Bar <a class="reference" href="#">bar</a>.</p>

        </div>

        </div>

        </div>

        <div class="halmos"></div>

        </div>

        </section>

        </div>

        </div>

        </body>
        """,
    )
    assert ":prev: references un-labeled step, link will not work" in caplog.text


def test_previous_outside_step():
    with pytest.raises(rsm.transformer.RSMTransformerError):
        compare_have_want(
            have="""\
            Foo :previous:1,bar::.
            """,
            want="X",
        )



"""Input: HTML body -- Output: WebManuscript."""

import logging
import re
from abc import ABC, abstractmethod
from pathlib import Path
from textwrap import dedent

from fs import open_fs
from fs.copy import copy_file

from .manuscript import WebManuscript

logger = logging.getLogger("RSM").getChild("build")


class BaseBuilder(ABC):
    """Use HTML body as a string and create a WebManuscript."""

    def __init__(self, asset_resolver=None, outname: str = "index.html", custom_css: str | None = None) -> None:
        self.body: str | None = None
        self.html: str | None = None
        self.web: WebManuscript | None = None
        self.outname: str = outname
        self.custom_css: str | None = custom_css
        # Default to disk-based asset resolver if none provided
        if asset_resolver is None:
            from .asset_resolver import AssetResolverFromDisk

            asset_resolver = AssetResolverFromDisk()
        self.asset_resolver = asset_resolver

    def build(self, body: str, src: Path = None) -> WebManuscript:
        logger.info("Building...")
        self.body = body
        self.web = WebManuscript(src)
        self.web.body = body

        logger.debug("Searching required static assets...")
        self.required_assets: list[Path] = []
        self.find_required_assets()

        logger.debug("Building main file...")
        self.make_main_file()
        return self.web

    @abstractmethod
    def make_main_file(self) -> None:
        pass

    def find_required_assets(self) -> None:
        self.required_assets = [
            Path(x) for x in re.findall(r'src="(.*?)"', str(self.body))
        ]


class HTMLBuilder(BaseBuilder):
    """Base class for builders that output HTML files.

    Produces a single index.html file with /static/ paths for resources.
    Subclasses can override make_html_header() to customize resource URLs.
    """

    body: str
    web: WebManuscript

    def make_main_file(self) -> None:
        html = str(
            "<html>\n\n"
            + self.make_html_header()
            + "\n"
            + self._inject_dark_mode_button(self.body.strip())
            + "\n\n"
            + self.make_html_footer()
            + "</html>\n"
        )
        self.web.writetext(self.outname, html)
        self.web.html = html

    def _inject_dark_mode_button(self, body: str) -> str:
        """Inject dark mode toggle button right after <body> tag."""
        button_html = dedent("""\
            <button class="dark-mode-toggle" onclick="toggleDarkMode()" aria-label="Toggle dark mode">
              <svg class="light-icon" xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
                <circle cx="12" cy="12" r="4"></circle>
                <path d="M12 2v2"></path>
                <path d="M12 20v2"></path>
                <path d="m4.93 4.93 1.41 1.41"></path>
                <path d="m17.66 17.66 1.41 1.41"></path>
                <path d="M2 12h2"></path>
                <path d="M20 12h2"></path>
                <path d="m6.34 17.66-1.41 1.41"></path>
                <path d="m19.07 4.93-1.41 1.41"></path>
              </svg>
              <svg class="dark-icon" xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
                <path d="M12 3a6 6 0 0 0 9 9 9 9 0 1 1 -9 -9"></path>
              </svg>
            </button>
        """)
        # Insert button right after <body> tag
        return body.replace("<body>", f"<body>\n\n{button_html}")


    def make_html_header(self) -> str:
        custom_css_link = ""
        if self.custom_css:
            custom_css_filename = Path(self.custom_css).name
            custom_css_link = f'  <link rel="stylesheet" type="text/css" href="/static/{custom_css_filename}" />\n'

        header = dedent(
            f"""\
        <head>
          <meta charset="utf-8" />
          <meta name="viewport" content="width=device-width, initial-scale=1.0" />
          <meta name="generator" content="RSM 0.0.1 https://github.com/leotrs/rsm" />

          <link rel="stylesheet" type="text/css" href="/static/rsm.css" />
          <link rel="stylesheet" type="text/css" href="/static/tooltipster.bundle.css" />
          <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/pseudocode@latest/build/pseudocode.min.css">
{custom_css_link}
          <script src="/static/jquery-3.6.0.js"></script>
          <script src="/static/tooltipster.bundle.js"></script>
          <script type="module">
            import {{ onload }} from '/static/onload.js';
            window.addEventListener('load', (ev) => {{window.lsp_ws = onload();}});
          </script>

          <script>
            // Dark mode toggle with localStorage and system preference detection
            (function() {{
              const savedTheme = localStorage.getItem('rsm-theme');
              const prefersDark = window.matchMedia('(prefers-color-scheme: dark)').matches;
              const isDark = savedTheme === 'dark' || (!savedTheme && prefersDark);

              if (isDark) {{
                document.documentElement.classList.add('dark-theme');
              }}
            }})();

            function toggleDarkMode() {{
              const isDark = document.documentElement.classList.toggle('dark-theme');
              localStorage.setItem('rsm-theme', isDark ? 'dark' : 'light');
            }}
          </script>

          <title>__TITLE_PLACEHOLDER__</title>
        </head>
        """
        )
        title = self._extract_title()
        header = header.replace("__TITLE_PLACEHOLDER__", title)
        return header

    def make_html_footer(self) -> str:
        return ""

    def _extract_title(self) -> str:
        mobj = re.search(r"<h1>(.*?)</h1>", self.body)
        if mobj is None:
            return ""
        else:
            return mobj.group(1)


class StandaloneBuilder(HTMLBuilder):
    """Builder that produces a single self-contained HTML file.

    Use this builder when the output file needs to work when opened directly
    in a browser (file:// URLs) without a web server. All RSM JavaScript is
    inlined directly in the HTML. Third-party libraries (jQuery, Tooltipster,
    MathJax) are loaded from CDNs.
    """

    CDN_JQUERY = "https://cdn.jsdelivr.net/npm/jquery@3.6.0/dist/jquery.min.js"
    CDN_TOOLTIPSTER_CSS = "https://cdn.jsdelivr.net/npm/tooltipster@4.2.8/dist/css/tooltipster.bundle.min.css"
    CDN_TOOLTIPSTER_JS = "https://cdn.jsdelivr.net/npm/tooltipster@4.2.8/dist/js/tooltipster.bundle.min.js"
    CDN_PSEUDOCODE_CSS = (
        "https://cdn.jsdelivr.net/npm/pseudocode@2.4.1/build/pseudocode.min.css"
    )
    CDN_RSM_CSS = "https://cdn.jsdelivr.net/gh/aris-pub/rsm@main/rsm/static/rsm.css"

    def _get_inline_js(self) -> str:
        """Read the bundled RSM JavaScript for inlining."""
        bundle_path = Path(__file__).parent / "static" / "rsm-standalone.js"
        return bundle_path.read_text()

    def _get_custom_css_inline(self) -> str:
        """Read custom CSS and return as inline <style> tag."""
        if not self.custom_css:
            return ""

        custom_css_path = Path(self.custom_css)
        if custom_css_path.exists():
            css_content = custom_css_path.read_text()
            return f"<style>\n{css_content}\n  </style>\n  "
        else:
            logger.warning(f"Custom CSS file not found: {self.custom_css}")
            return ""

    def make_html_header(self) -> str:
        inline_js = self._get_inline_js()
        custom_css_inline = self._get_custom_css_inline()
        header = f"""\
<head>
  <meta charset="utf-8" />
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  <meta name="generator" content="RSM 0.0.1 https://github.com/leotrs/rsm" />

  <link rel="stylesheet" type="text/css" href="{self.CDN_RSM_CSS}" />
  <link rel="stylesheet" type="text/css" href="{self.CDN_TOOLTIPSTER_CSS}" />
  <link rel="stylesheet" href="{self.CDN_PSEUDOCODE_CSS}">
  {custom_css_inline}
  <script src="{self.CDN_JQUERY}"></script>
  <script src="{self.CDN_TOOLTIPSTER_JS}"></script>
  <script>
{inline_js}
  </script>
  <script>
    window.addEventListener('load', function() {{ RSM.onload(null, {{keys: false}}); }});
  </script>

  <script>
    // Dark mode toggle with localStorage and system preference detection
    (function() {{
      const savedTheme = localStorage.getItem('rsm-theme');
      const prefersDark = window.matchMedia('(prefers-color-scheme: dark)').matches;
      const isDark = savedTheme === 'dark' || (!savedTheme && prefersDark);

      if (isDark) {{
        document.documentElement.classList.add('dark-theme');
      }}
    }})();

    function toggleDarkMode() {{
      const isDark = document.documentElement.classList.toggle('dark-theme');
      localStorage.setItem('rsm-theme', isDark ? 'dark' : 'light');
    }}
  </script>

  <title>__TITLE_PLACEHOLDER__</title>
</head>
"""
        title = self._extract_title()
        header = header.replace("__TITLE_PLACEHOLDER__", title)
        return header


class FolderBuilder(HTMLBuilder):
    """Builder that outputs an HTML file plus a static/ folder with all assets."""

    def build(self, body: str, src: Path = None) -> WebManuscript:
        super().build(body, src)
        logger.debug("Moving default RSM assets...")
        self.mount_static()
        if self.required_assets:
            logger.debug("Moving user assets...")
            self.mount_required_assets()
        return self.web

    def mount_static(self) -> None:
        working_path = Path(__file__).parent.absolute()
        source_path = (working_path / "static").resolve()
        source = open_fs(str(source_path))

        self.web.makedir("static")
        for fn in [
            fn for fn in source.listdir(".") if Path(fn).suffix in {".js", ".css"}
        ]:
            copy_file(source, fn, self.web, f"static/{fn}")

        # Copy custom CSS if provided
        if self.custom_css:
            custom_css_path = Path(self.custom_css)
            if custom_css_path.exists():
                self.web.writetext(
                    f"static/{custom_css_path.name}",
                    custom_css_path.read_text()
                )
            else:
                logger.warning(f"Custom CSS file not found: {self.custom_css}")

    def mount_required_assets(self) -> None:
        """Mount required assets using the asset resolver.

        Technical Debt Note: This method demonstrates the architectural inconsistency
        where assets use the resolver system but the main RSM file still uses
        direct filesystem access via Reader class. This should be unified in future
        refactoring.
        """
        for fn in self.required_assets:
            asset_content = self.asset_resolver.resolve_asset(str(fn))
            if asset_content is not None:
                self.web.writetext(f"static/{fn.name}", asset_content)
            else:
                logger.warning(f"Unable to resolve required asset: {fn}")
